#include "headers.h"

MODULE = Arisa	PACKAGE = Arisa

PROTOTYPES: ENABLE

BOOT:
	arisa_boot(Network);
	arisa_boot(Channel);
	arisa_boot(Interface);
	arisa_boot(Pool);
	arisa_boot(Queue);
	arisa_boot(Packlist);
	arisa_boot(User);
	arisa_boot(Admin);
	arisa_boot(UI);

void
log(msg = NULL)
	char *msg
CODE:
	if(msg != NULL)
		LOGP(L_INF,"%s",msg);


SV*
ignore_add(...)
CODE:
	RETVAL = perl_ignore_add(sp,mark,ax,items,global->ignore_list);
OUTPUT:
	RETVAL

SV*
ignore_del(...)
CODE:
	RETVAL = perl_ignore_del(sp,mark,ax,items,global->ignore_list);	
OUTPUT:
	RETVAL

void
ignore_list(...)
PREINIT:
	int i;
PPCODE:
	LOCK(global->ignore_list);
	for(i = 0; i < global->ignore_list->no_entries; ++i) {
		XPUSHs(sv_2mortal(newRV_noinc(
			(SV*)ignore_entry_to_hv(
				global->ignore_list->entries[i]))));
	}
	UNLOCK(global->ignore_list);

SV*
ignore_move(src, dst)
	int src
	int dst
CODE:
	RETVAL = perl_ignore_move(global->ignore_list,src,dst);
OUTPUT:
	RETVAL

void
ignore_settings(...)
PREINIT:
	HV *hv;
PPCODE:
	hv = perl_ignore_settings(sp,mark,ax,items,global->ignore_list);
	RET_REF(hv);

SV*
ignore_test(host)
	char *host
CODE:
	RETVAL = &PL_sv_no;
	if(host != NULL) {
		if(ignore_test(global->ignore_list,host))
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL
	
void
ignore_wipe()
CODE:
	ignore_wipe(global->ignore_list);

void
network_defaults(...)
PREINIT:
	pqueue_t settings;
	HV *hv = newHV();
PPCODE:
	LOCK(global);
	if((items % 2) == 0) {
		STRLEN n_a;
		int i;
		for(i = 0; i < items; i += 2) {
			char *name = (char *)SvPV(ST(i), n_a);
			char *value = (char *)SvPV(ST(i+1), n_a);
			netset_apply(global->network_settings,name,value);
		}
	}
	pqueue_init(&settings,0);
	netset_read(global->network_settings,&settings);
	UNLOCK(global);
	hv_store_values(hv,&settings);
	RET_REF(hv);	

void
request_hash(file)
	char *file
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(file != NULL) {
		pack_t *p = pack_cache_get(global->pack_cache,file,0);
		if(p != NULL) {
			RLOCK(p);
			if(p->hashed != 0)
				hv = pack_to_hv(p);
			RUNLOCK(p);
			pack_deref(p);
		}
		if(hv == NULL) {
			p 	= alloc_pack();
			p->file	= xstrdup(file);
			if(hash_request(p) == 0)
				hv = pack_to_hv(p);
			free_pack(p);
		}
	}
	RET_REF(hv);
	
void
script_autoload(...)
CODE:
	LOCK(global->settings);
	if(items > 0) {
		av_to_strarr(&(global->settings->scripts),
			&(global->settings->no_scripts),ST(0));
	}
	RET_STRARR(global->settings->scripts,global->settings->no_scripts);
	UNLOCK(global->settings);
	
SV*
script_load(file)
	char *file
CODE:
	RETVAL = &PL_sv_no;
	if(file != NULL) {
		if(se_load_script(file) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
script_reload(file)
	char *file
CODE:
	RETVAL = &PL_sv_no;
	if(file != NULL) {
		if(se_reload_script(file) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL
	
SV*
script_unload(file)
	char *file
CODE:
	RETVAL = &PL_sv_no;
	if(file != NULL) {
		if(se_reload_script(file) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

void
scripting_engines()
PREINIT:
	int i;
PPCODE:
	LOCKR(&global->se.lock);
	for(i = 0; i < global->se.no_interpreters; ++i) {
		interpreter_t *inp = global->se.interpreters[i];
		HV *hv 		= newHV();
		AV *types 	= newAV();
		AV *loaded 	= newAV();
		int j;

		LOCK(inp);
		for(j = 0; j < inp->no_types; ++j)
			av_push(types,new_pv(inp->types[j]));
		for(j = 0; j < inp->no_loaded; ++j)
			av_push(loaded,new_pv(inp->loaded[j]));
		hv_store(hv,"name",4,new_pv(inp->name),0);
		hv_store(hv,"types",5,newRV_noinc((SV*)types),0);
		hv_store(hv,"loaded",6,newRV_noinc((SV*)loaded),0);
		UNLOCK(inp);

		XPUSHs(sv_2mortal(newRV_noinc((SV*)hv)));
	}
	UNLOCKR(&global->se.lock);
	
void
settings(...)
PREINIT:
	pqueue_t settings;
	HV *hv = newHV();
PPCODE:
	if((items % 2) == 0) {
		STRLEN n_a;
		int i;
		for(i = 0; i < items; i += 2) {
			char *name = (char *)SvPV(ST(i), n_a);
			char *value = (char *)SvPV(ST(i+1), n_a);
			global_apply_setting(name,value);
		}
	}
	pqueue_init(&settings,0);
	global_read_settings(&settings);
	hv_store_values(hv,&settings);
	RET_REF(hv);

SV*
shutdown(...)
PREINIT:
	char	*type;
	time_t	when;
CODE:
	RETVAL = &PL_sv_no;
	type = SvPVarg("type");
	when = SvUVarg("when");
	if(type != NULL) {
		if(strcasecmp(type,"now") == 0) {
			arisa_signal_end();
			RETVAL = &PL_sv_yes;
		} else if(strcasecmp(type,"cancel") == 0) {
			LOCK(global);
			global->end = 0;
			global->end_time = 0;
			UNLOCK(global);
			RETVAL = &PL_sv_yes;
		} else if(strcasecmp(type,"delayed") == 0) {
			LOCK(global);
			global->end_time = TIME_MAX;
			global->end = 1;
			UNLOCK(global);
			RETVAL = &PL_sv_yes;
		} else if(strcasecmp(type,"timed") == 0 && when != 0) {
			LOCK(global);
			global->end_time = when;
			global->end = 1;
			UNLOCK(global);
			RETVAL = &PL_sv_yes;
		} else {
			LOCK(global);
			if(global->end_time == TIME_MAX) {
				RETVAL = new_pv("delayed");
			} else if(global->end_time != 0) {
				RETVAL = newSVuv(global->end_time);
			}
			UNLOCK(global);
		}
	}
OUTPUT:
	RETVAL
